/***************************************************************************
 *   Copyright (C) 2008 by Daniel Calviño Sánchez                          *
 *   danxuliu@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include <QtTest>

#define protected public
#define private public
#include "Step.h"
#undef private
#undef protected

#include "Option.h"
#include "WaitForSignal.h"

class StepTest: public QObject {
Q_OBJECT

public slots:

    void dummySlot() {
        mDummySlotCallCount++;
    }

signals:

    void dummySignal();

    void anotherDummySignal();

private:

    int mDummySlotCallCount;

private slots:

    void init() {
        mDummySlotCallCount = 0;
    }

    void testAddOption();

    void testAddWaitFor();

    void testRemoveOption();

    void testRemoveWaitFor();

};

void StepTest::testAddOption() {
    Step step("doSomethingConstructive");

    //Add a first option
    Option* option1 = new Option("Bathe your iguana");

    step.addOption(option1, this, SLOT(dummySlot()));
    connect(this, SIGNAL(dummySignal()), option1, SIGNAL(selected()));

    QCOMPARE(option1->parent(), &step);
    QCOMPARE(step.options().count(), 1);
    QVERIFY(step.options().contains(option1));
    QCOMPARE(mDummySlotCallCount, 0);

    emit dummySignal();
    QCOMPARE(mDummySlotCallCount, 0);

    step.setActive(true);
    emit dummySignal();
    QCOMPARE(mDummySlotCallCount, 1);

    //Add a second option
    Option* option2 = new Option("Feed a toucan");

    step.addOption(option2, this, SLOT(dummySlot()));

    QCOMPARE(option2->parent(), &step);
    QCOMPARE(step.options().count(), 2);
    QVERIFY(step.options().contains(option1));
    QVERIFY(step.options().contains(option2));
    QCOMPARE(mDummySlotCallCount, 1);

    //Tests with wrong values

    //Add the same option again
    step.addOption(option2, this, SLOT(dummySlot()));

    QCOMPARE(option2->parent(), &step);
    QCOMPARE(step.options().count(), 2);
    QVERIFY(step.options().contains(option1));
    QVERIFY(step.options().contains(option2));

    //Add a different option with the same name
    Option option3("Bathe your iguana");

    step.addOption(&option3, this, SLOT(dummySlot()));

    QCOMPARE(option3.parent(), (QObject*)0);
    QCOMPARE(step.options().count(), 2);
    QVERIFY(step.options().contains(option1));
    QVERIFY(step.options().contains(option2));
}

void StepTest::testAddWaitFor() {
    Step step("someName");

    //Add a first condition
    WaitFor* waitFor1 = new WaitForSignal(this, SIGNAL(dummySignal()));

    step.addWaitFor(waitFor1, this, SLOT(dummySlot()));

    QCOMPARE(waitFor1->parent(), &step);
    QCOMPARE(step.mWaitsFor.count(), 1);
    QVERIFY(step.mWaitsFor.contains(waitFor1));
    QCOMPARE(mDummySlotCallCount, 0);

    emit dummySignal();
    QCOMPARE(mDummySlotCallCount, 0);

    step.setActive(true);
    emit dummySignal();
    QCOMPARE(mDummySlotCallCount, 1);

    //Add a second condition
    WaitFor* waitFor2 = new WaitForSignal(this, SIGNAL(anotherDummySignal()));

    step.addWaitFor(waitFor2, this, SLOT(dummySlot()));

    QCOMPARE(waitFor2->parent(), &step);
    QCOMPARE(step.mWaitsFor.count(), 2);
    QVERIFY(step.mWaitsFor.contains(waitFor1));
    QVERIFY(step.mWaitsFor.contains(waitFor2));
    QCOMPARE(mDummySlotCallCount, 1);

    emit dummySignal();
    QCOMPARE(mDummySlotCallCount, 2);

    //Tests with wrong values

    //Add the same condition again
    step.addWaitFor(waitFor2, this, SLOT(dummySlot()));

    QCOMPARE(waitFor2->parent(), &step);
    QCOMPARE(step.mWaitsFor.count(), 2);
    QVERIFY(step.mWaitsFor.contains(waitFor1));
    QVERIFY(step.mWaitsFor.contains(waitFor2));
}

void StepTest::testRemoveOption() {
    Step step("doSomethingConstructive");

    //They will be removed and not deleted by parent Step, so they are created
    //in stack
    Option option1("Bathe your iguana");
    step.addOption(&option1, this, SLOT(dummySlot()));
    connect(this, SIGNAL(dummySignal()), &option1, SIGNAL(selected()));

    Option option2("Feed a toucan");
    step.addOption(&option2, this, SLOT(dummySlot()));
    connect(this, SIGNAL(dummySignal()), &option2, SIGNAL(selected()));

    step.setActive(true);
    emit dummySignal();
    QCOMPARE(mDummySlotCallCount, 2);

    //Remove a first option
    step.removeOption(&option1);

    QCOMPARE(option1.parent(), (QObject*)0);
    QCOMPARE(step.options().count(), 1);
    QVERIFY(step.options().contains(&option2));
    QVERIFY(!step.options().contains(&option1));
    QCOMPARE(mDummySlotCallCount, 2);

    emit dummySignal();
    QCOMPARE(mDummySlotCallCount, 3);

    //Remove a second option
    step.removeOption(&option2);

    QCOMPARE(option1.parent(), (QObject*)0);
    QCOMPARE(step.options().count(), 0);
    QVERIFY(!step.options().contains(&option2));
    QCOMPARE(mDummySlotCallCount, 3);
}

void StepTest::testRemoveWaitFor() {
    Step step("someName");

    //They will be removed and not deleted by parent Step, so they are created
    //in stack
    WaitForSignal waitFor1(this, SIGNAL(dummySignal()));
    step.addWaitFor(&waitFor1, this, SLOT(dummySlot()));

    WaitForSignal waitFor2(this, SIGNAL(anotherDummySignal()));
    step.addWaitFor(&waitFor2, this, SLOT(dummySlot()));

    step.setActive(true);
    emit dummySignal();
    emit anotherDummySignal();
    QCOMPARE(mDummySlotCallCount, 2);

    //Remove a condition
    step.removeWaitFor(&waitFor1);

    QCOMPARE(waitFor1.parent(), (QObject*)0);
    QCOMPARE(step.mWaitsFor.count(), 1);
    QVERIFY(step.mWaitsFor.contains(&waitFor2));
    QVERIFY(!step.mWaitsFor.contains(&waitFor1));
    QVERIFY(!waitFor1.isActive());
    QCOMPARE(mDummySlotCallCount, 2);

    emit dummySignal();
    emit anotherDummySignal();
    QCOMPARE(mDummySlotCallCount, 3);

    //Remove a second condition
    step.removeWaitFor(&waitFor2);

    QCOMPARE(waitFor2.parent(), (QObject*)0);
    QCOMPARE(step.mWaitsFor.count(), 0);
    QVERIFY(!step.mWaitsFor.contains(&waitFor2));
    QVERIFY(!waitFor2.isActive());

    emit dummySignal();
    emit anotherDummySignal();
    QCOMPARE(mDummySlotCallCount, 3);
}

QTEST_MAIN(StepTest)

#include "StepTest.moc"
